/**
  * @file SignalRelayFT90x.c
  * @brief This module contains the implementation of the signal relay hardware
  * functions.
  *
  * @author Stefan Popovic
  *
  * @date 14/10/2016
  *
  * @version 0.1a
  *
  * @copyright GNU Public License
  *
  * Test configuration:
  *
  * MCU: FT900
  * Dev.Board: EasyFT90x v7
  * External Clock 72.0000 MHz
  * Ext. Modules: Signal Relay Click Board
  * SW: MikroC PRO for FT90x v2.0.0
  *
  * NOTES:
    - Signal Relay click board is essentially board with 4 5VDC Signal Relays
    - Place Signal Relay click board on mikroBUS slot 1
    - Pull-up switch SW1.7 and SW1.8 (VCC position) and
    - Pull-down GPIO_PORT_48_55.B1, GPIO_PORT_48_55.B2, GPIO_PORT_56_63.B1 
      and GPIO_PORT_56_63.B2
    - RE 1: GP09
    - RE 2: GP01
    - RE 3: GP28
    - RE 4: GP56


  /**************************************************************************
  * Module Preprocessor Constants
  **************************************************************************/


  /**************************************************************************
  * Module Preprocessor Macros
  **************************************************************************/


  /**************************************************************************
  * Module Typedefs
  **************************************************************************/


   /**************************************************************************
   * Includes
   **************************************************************************/
   #include "built_in.h"
   #include "recources.h"
   #include <stdbool.h>

  /**************************************************************************
  * Module Variable Definitions
  **************************************************************************/

  char txt[24];
  bool oldGP49;
  bool oldGP50;
  bool oldGP57;
  bool oldGP58;

  // TFT module connections
  char TFT_DataPort at GPIO_PORT_32_39;
  sbit TFT_RST at GPIO_PIN42_bit;
  sbit TFT_RS at GPIO_PIN40_bit;
  sbit TFT_CS at GPIO_PIN41_bit;
  sbit TFT_RD at GPIO_PIN46_bit;
  sbit TFT_WR at GPIO_PIN47_bit;
  // End TFT module connections

   void Display_Init()
  {
    TFT_Init_ILI9341_8bit(320, 240); // TFT Initialization
  }

  // Draw Frame on TFT module
  void DrawFrame()
  {
    TFT_Fill_Screen(CL_WHITE);
    TFT_Set_Pen(CL_BLACK, 1);
    TFT_Set_Brush(1,CL_BLACK, 0, 0, CL_BLACK, CL_BLACK);
    TFT_Line(20, 220, 300, 220);
    TFT_Line(5,  46, 315,  46);
    TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_BLACK, FO_HORIZONTAL);
    TFT_Write_Text("Signal Relay  Click  Board  Example", 5, 14);
    TFT_Set_Font(&Verdana12x13_Regular, CL_BLACK, FO_HORIZONTAL);
    TFT_Write_Text("EasyPIC Fusion v7 for P32", 19, 223);
    TFT_Set_Font(&Verdana12x13_Regular, CL_BLACK, FO_HORIZONTAL);
    TFT_Write_Text("www.mikroe.com", 200, 223);

    TFT_Write_Text("RELAY 1:", 10, 90 );
    TFT_Write_Text("RELAY 2:", 170, 90 );
    TFT_Write_Text("RELAY 3:", 10, 150);
    TFT_Write_Text("RELAY 4:", 170, 150 );

    TFT_Set_Pen(CL_RED, 5);
    TFT_Set_Brush(1, CL_RED, 0, 0, CL_RED, CL_RED);

    // Switch for RELAY 1
    TFT_Line(80, 100, 100, 100);
    TFT_Line(130, 100, 150, 100);
    TFT_Line(100, 100, 130, 80);     // OPEN switch RE1

    // Switch for RELAY 2
    TFT_Line(240, 100, 260, 100);
    TFT_Line(290, 100, 310, 100);
    TFT_Line(260, 100, 290, 80);     // OPEN switch RE2

    // Switch for RELAY 3
    TFT_Line(80, 155, 100, 155);
    TFT_Line(130, 155, 150, 155);
    TFT_Line(100, 155, 130, 135);    // OPEN switch RE3

    // Switch for RELAY 4
    TFT_Line(240, 155, 260, 155);
    TFT_Line(290, 155, 310, 155);
    TFT_Line(260, 155, 290, 135);    // OPEN switch RE4

  }
  void main()
  {
    Display_Init();
    DrawFrame();

    oldGP50 = false;
    oldGP49= false;
    oldGP57 = false;
    oldGP58 = false;
    
    // Setting pins GPIO49, GPIO50, GPIO57, GPIO58 as digital input
    GPIO_Digital_Input(&GPIO_PORT_48_55, 
                       _GPIO_PINMASK_1 | 
                       _GPIO_PINMASK_1);
    GPIO_Digital_Input(&GPIO_PORT_56_63, 
                       _GPIO_PINMASK_1 
                       _GPIO_PINMASK_2);

    // Setting pins  GPIO1, GPIO9, GPIO28 and GPIO56 as digital output
    GPIO_Digital_Output(&GPIO_PORT_00_07, _GPIO_PINMASK_1);    
    GPIO_Digital_Output(&GPIO_PORT_08_15, _GPIO_PINMASK_1);    
    GPIO_Digital_Output(&GPIO_PORT_24_31, _GPIO_PINMASK_4);    
    GPIO_Digital_Output(&GPIO_PORT_56_63, _GPIO_PINMASK_0);    
    
    //Initializing port values to zero
    GPIO_PORT_00_07 = 0x00;
    GPIO_PORT_08_15 = 0x00;
    GPIO_PORT_24_31 = 0x00;
    GPIO_PORT_56_63 = 0x00;
    

    while(1)                         // endless loop
    {

        // Relay 1
        if (Button(&GPIO_PORT_48_55, 2, 1, 1))   // detect logical one on GP50 pin
          {

            if (!oldGP50)                        // detect zero-to-one transition
            {
              Delay_ms(300);
              //marking
              TFT_Set_Pen(CL_WHITE, 5);
              TFT_Set_Brush(1, CL_WHITE, 0, 0, CL_WHITE, CL_WHITE);
              TFT_Line(100, 100, 130, 80);       // OPEN switch

              //showing
              TFT_Set_Pen(CL_RED, 5);
              TFT_Set_Brush(1, CL_RED, 0, 0, CL_RED, CL_RED);

              TFT_Line(100, 100, 130, 100);      // CLOSED switch

              oldGP50 = 1;

              GPIO_PORT_08_15 = GPIO_PORT_08_15 | 0x02;

              }
            else                                 // detect one-to-zero transition
            {
              Delay_ms(300);
              //marking
              TFT_Set_Pen(CL_WHITE, 5);
              TFT_Set_Brush(1, CL_WHITE, 0, 0, CL_WHITE, CL_WHITE);
              TFT_Line(100, 100, 130, 100);      // CLOSED switch

              //showing
              TFT_Set_Pen(CL_RED, 5);
              TFT_Set_Brush(1, CL_RED, 0, 0, CL_RED, CL_RED);
              TFT_Line(100, 100, 130, 80);       // OPEN switch

              oldGP50 = 0;

              GPIO_PORT_08_15 = GPIO_PORT_08_15 & 0xFD;
            }
        }

        // Relay 2
        if (Button(&GPIO_PORT_56_63, 2, 1, 1))   // detect logical one on GP58 pin
          {

            if (!oldGP58)                        // detect zero-to-one transition
            {
              Delay_ms(300);
              // marking
              TFT_Set_Pen(CL_WHITE, 5);
              TFT_Set_Brush(1, CL_WHITE, 0, 0, CL_WHITE, CL_WHITE);
              TFT_Line(260, 100, 290, 80);       // OPEN switch

              // showing
              TFT_Set_Pen(CL_RED, 5);
              TFT_Set_Brush(1, CL_RED, 0, 0, CL_RED, CL_RED);

              TFT_Line(260, 100, 290, 100);      // CLOSED switch

              oldGP58 = 1;

              GPIO_PORT_00_07 = GPIO_PORT_00_07 | 0x02;

            }
            else                                 // detect one-to-zero transition
            {
              Delay_ms(300);
              // marking
              TFT_Set_Pen(CL_WHITE, 5);
              TFT_Set_Brush(1, CL_WHITE, 0, 0, CL_WHITE, CL_WHITE);
              TFT_Line(260, 100, 290, 100);      // CLOSED switch

              // showing
              TFT_Set_Pen(CL_RED, 5);
              TFT_Set_Brush(1, CL_RED, 0, 0, CL_RED, CL_RED);
              TFT_Line(260, 100, 290, 80);       // OPEN switch

              oldGP58 = 0;

              GPIO_PORT_00_07 = GPIO_PORT_00_07 & 0xFD;
            }
          }

        // Relay 3
        if (Button(&GPIO_PORT_48_55, 1, 1, 1))   // detect logical one on GP49 pin
          {

            if (!oldGP49)                        // detect zero-to-one transition
            {
              Delay_ms(300);
              //marking
              TFT_Set_Pen(CL_WHITE, 5);
              TFT_Set_Brush(1, CL_WHITE, 0, 0, CL_WHITE, CL_WHITE);
              TFT_Line(100, 155, 130, 135);      // OPEN switch

              //showing
              TFT_Set_Pen(CL_RED, 5);
              TFT_Set_Brush(1, CL_RED, 0, 0, CL_RED, CL_RED);

              TFT_Line(100, 155, 130, 155);      // CLOSED switch

              oldGP49 = 1;

              GPIO_PORT_24_31 = GPIO_PORT_24_31 | 0x10;
              

            }
            else                                 // detect one-to-zero transition
            {
              Delay_ms(300);
              //marking
              TFT_Set_Pen(CL_WHITE, 5);
              TFT_Set_Brush(1, CL_WHITE, 0, 0, CL_WHITE, CL_WHITE);
              TFT_Line(100, 155, 130, 155);      // CLOSED switch

              //showing
              TFT_Set_Pen(CL_RED, 5);
              TFT_Set_Brush(1, CL_RED, 0, 0, CL_RED, CL_RED);
              TFT_Line(100, 155, 130, 135);      // OPEN switch

              oldGP49 = 0;

              GPIO_PORT_24_31 = GPIO_PORT_24_31 & 0xEF;
            }
          }

        // Relay 4
        if (Button(&GPIO_PORT_56_63, 1, 1, 1))   // detect logical one on GP57 pin
          {

            if (!oldGP57)                        // detect zero-to-one transition
            {
              Delay_ms(300);
              //marking
              TFT_Set_Pen(CL_WHITE, 5);
              TFT_Set_Brush(1, CL_WHITE, 0, 0, CL_WHITE, CL_WHITE);
              TFT_Line(260, 155, 290, 135);      // OPEN switch

              //showing
              TFT_Set_Pen(CL_RED, 5);
              TFT_Set_Brush(1, CL_RED, 0, 0, CL_RED, CL_RED);

              TFT_Line(260, 155, 290, 155);      // CLOSED switch

              oldGP57 = 1;

              GPIO_PORT_56_63 = GPIO_PORT_56_63 | 0x01;

            }
            else                                 // detect one-to-zero transition
            {
              Delay_ms(300);
              //marking
              TFT_Set_Pen(CL_WHITE, 5);
              TFT_Set_Brush(1, CL_WHITE, 0, 0, CL_WHITE, CL_WHITE);
              TFT_Line(260, 155, 290, 155);      // CLOSED switch

              //showing
              TFT_Set_Pen(CL_RED, 5);
              TFT_Set_Brush(1, CL_RED, 0, 0, CL_RED, CL_RED);
              TFT_Line(260, 155, 290, 135);      // OPEN switch

              oldGP57 = 0;

              GPIO_PORT_56_63 = GPIO_PORT_56_63 & 0xFE;
            }
          }
    }
  }